#include "hal_data.h"
#include "coaster.h"

adc_data_size_t light_reading;
adc_data_size_t ambient_light;
uint8_t first_reading = 1;
uint8_t LED_on_flag = 0;

extern uint8_t play_song_flag;  // flag to indicate if song is playing
uint8_t ambient_light_set_flag = 0; // flag to set ambient light reading after a brief pause on power up
extern uint8_t ADC_Data_ready, ADC_Timer_started;

ioport_level_t level = IOPORT_LEVEL_LOW;
ioport_port_pin_t LED_pin[5][3];
timer_size_t blink_rate = 1000;
float shadow_ratio;
uint8_t blink_on = 0;
uint8_t justplayed = 0;
uint8_t lowbattdetect = 0;
uint8_t coldness = 0;
uint8_t lowbatt_check = 0; // index for low battery detect
uint8_t lowbatt = 0;  // flag for low battery detect

void blink_LEDs(void)
{
    uint8_t status;
    // LED_pin[LED#][color] colors are 0=Red, 1=Green, 2=Blue.
    // arranged in order from bottom representind LED[0] and then going clockwise
    LED_pin[0][2] = IOPORT_PORT_04_PIN_00;
    LED_pin[0][1] = IOPORT_PORT_04_PIN_01;
    LED_pin[0][0] = IOPORT_PORT_04_PIN_02;
    LED_pin[1][2] = IOPORT_PORT_01_PIN_00;
    LED_pin[1][1] = IOPORT_PORT_01_PIN_01;
    LED_pin[1][0] = IOPORT_PORT_01_PIN_02;
    LED_pin[2][2] = IOPORT_PORT_01_PIN_03;
    LED_pin[2][1] = IOPORT_PORT_01_PIN_04;
    LED_pin[2][0] = IOPORT_PORT_01_PIN_05;
    LED_pin[3][2] = IOPORT_PORT_03_PIN_02;
    LED_pin[3][1] = IOPORT_PORT_03_PIN_03;
    LED_pin[3][0] = IOPORT_PORT_03_PIN_04;
    LED_pin[4][2] = IOPORT_PORT_04_PIN_07;
    LED_pin[4][1] = IOPORT_PORT_04_PIN_08;
    LED_pin[4][0] = IOPORT_PORT_04_PIN_09;

    extern float temperature, ambient_temp;


    // initialize and start the ambient room light timer
    status = ambient_timer.p_api -> open(ambient_timer.p_ctrl, ambient_timer.p_cfg);
    APP_ERR_TRAP(status);
    status = ambient_timer.p_api ->reset(ambient_timer.p_ctrl);
    APP_ERR_TRAP(status);
    status = ambient_timer.p_api -> start(ambient_timer.p_ctrl);
    APP_ERR_TRAP(status);

    // initialize and start the blink_rate_timer
    status = blink_rate_timer.p_api -> open(blink_rate_timer.p_ctrl, blink_rate_timer.p_cfg);
    APP_ERR_TRAP(status);
    status = blink_rate_timer.p_api ->reset(blink_rate_timer.p_ctrl);
    APP_ERR_TRAP(status);
    status = blink_rate_timer.p_api -> start(blink_rate_timer.p_ctrl);
    APP_ERR_TRAP(status);

    // Initialize song timers
    status = g_playnote_timer.p_api->open(g_playnote_timer.p_ctrl,g_playnote_timer.p_cfg);
    APP_ERR_TRAP(status);
    status = g_notesdur_timer.p_api->open(g_notesdur_timer.p_ctrl, g_notesdur_timer.p_cfg);
    APP_ERR_TRAP(status);
    status = g_notepause_timer.p_api->open(g_notepause_timer.p_ctrl,g_notepause_timer.p_cfg);
    APP_ERR_TRAP(status);
    // Initialize ADC
    ADC_Data_ready = 0;
    ADC_Timer_started = 0;

    adc_init();
    /*
    * Uncomment this section if U1 is installed.   Connect the Lowbatt line to P206 and add
    * an external IRQ with a callback.


    // enable Low Battery Detect interrupt
    g_lowbatt_irq0.p_api->open(g_lowbatt_irq0.p_ctrl, g_lowbatt_irq0.p_cfg);
    g_lowbatt_irq0.p_api->enable(g_lowbatt_irq0.p_ctrl);
    g_lowbatt_irq0.p_api->triggerSet(g_lowbatt_irq0.p_ctrl, EXTERNAL_IRQ_TRIG_RISING );
     */
    // initialize the notes of the song
    play_song_init();


    // Main Loop
    while(1)
    {
        if(ADC_Data_ready){
            read_adc_results();
        }

        if(ADC_Timer_started) {
            adc_start();
            ADC_Timer_started = 0;
        }

        if (play_song_flag == 0)
        {
            // Stop Song timers
            status = g_playnote_timer.p_api->stop(g_playnote_timer.p_ctrl);
            APP_ERR_TRAP(status);
            status = g_notesdur_timer.p_api->stop(g_notesdur_timer.p_ctrl);
            APP_ERR_TRAP(status);

            if ((temperature - ambient_temp) >= 1.2)
            {
                coldness = 0;  // hot condition
            }
            else if ( ((temperature - ambient_temp) > 0.6) && ((temperature - ambient_temp) < 1.2))
            {
                coldness = 1;  // warming condition
            }
            else
            {
                coldness = 2;  // cold condition
            }

            // Calculate shadow to ambient light ratio to be used to detect presence of beverage
            shadow_ratio = ((float)ambient_light -(float)light_reading) / (float)ambient_light;

            if (shadow_ratio <= MIN_DC )
            {
                LED_on_flag = 0;  // object is too far away
                for (int x = 0; x<5; x++)
                {
                    for (int y = 0; y<3; y++)
                    {
                        g_ioport.p_api->pinWrite(LED_pin[x][y], IOPORT_LEVEL_HIGH);  // turn off all LEDs
                    }
                }
                justplayed = 0;
            }
            if ((shadow_ratio > MIN_DC && shadow_ratio < MAX_DC) && justplayed == 0) // beverage presence has been detected
            {
                if (blink_on == 1)
                {
                    level ^= 1;
                    blink_on = 0;

                    for (int x = 0; x<5; x++)
                    {
                        if (coldness == 0)  // hot condition -- blink red LEDs
                        {
                            g_ioport.p_api->pinWrite(LED_pin[x][0], level);  // blink red LEDs
                            g_ioport.p_api->pinWrite(LED_pin[x][1], IOPORT_LEVEL_HIGH);  // turn off green LEDs
                            g_ioport.p_api->pinWrite(LED_pin[x][2], IOPORT_LEVEL_HIGH);  // turn off blue LEDs
                        }
                        else if (coldness == 1)  // warming condition -- blink purple LEDs
                        {
                            g_ioport.p_api->pinWrite(LED_pin[x][0], level);  // blink red LEDs
                            g_ioport.p_api->pinWrite(LED_pin[x][1], IOPORT_LEVEL_HIGH);  // turn off green LEDs
                            g_ioport.p_api->pinWrite(LED_pin[x][2], level);  // also blink blue LEDs to create purple
                        }
                        else  // cold condition -- blink blue
                        {
                            g_ioport.p_api->pinWrite(LED_pin[x][0], IOPORT_LEVEL_HIGH);  // turn off red LEDs
                            g_ioport.p_api->pinWrite(LED_pin[x][1], IOPORT_LEVEL_HIGH);  // turn off green LEDs
                            g_ioport.p_api->pinWrite(LED_pin[x][2], level);  // blink blue LEDs
                        }
                    }


                    // update LED flash rate
                    blink_rate = (timer_size_t)((290/(MIN_DC - MAX_DC)) * shadow_ratio + 300 - ((MIN_DC * 290)/(MIN_DC - MAX_DC)));
                    status = blink_rate_timer.p_api ->stop(blink_rate_timer.p_ctrl);
                    APP_ERR_TRAP(status);
                    status = blink_rate_timer.p_api ->reset(blink_rate_timer.p_ctrl);
                    APP_ERR_TRAP(status);
                    status = blink_rate_timer.p_api -> periodSet(blink_rate_timer.p_ctrl, blink_rate, TIMER_UNIT_PERIOD_MSEC);
                    APP_ERR_TRAP(status);
                    status = blink_rate_timer.p_api -> start(blink_rate_timer.p_ctrl);
                    APP_ERR_TRAP(status);

                }
            }
            if (shadow_ratio >= MAX_DC)
            {
                LED_on_flag = 1;
                // beverage is resting on coaster
                for (int x = 0; x<5; x++)
                {
                    if (coldness == 0)  // hot condition --  red LEDs
                    {
                        g_ioport.p_api->pinWrite(LED_pin[x][0], IOPORT_LEVEL_LOW);  // turn on red LEDs
                        g_ioport.p_api->pinWrite(LED_pin[x][1], IOPORT_LEVEL_HIGH);  // turn off green LEDs
                        g_ioport.p_api->pinWrite(LED_pin[x][2], IOPORT_LEVEL_HIGH);  // turn off blue LEDs
                    }
                    else if (coldness == 1)  // warming condition -- purple LEDs
                    {
                        g_ioport.p_api->pinWrite(LED_pin[x][0], IOPORT_LEVEL_LOW);  // turn on red LEDs
                        g_ioport.p_api->pinWrite(LED_pin[x][1], IOPORT_LEVEL_HIGH);  // turn off green LEDs
                        g_ioport.p_api->pinWrite(LED_pin[x][2], IOPORT_LEVEL_LOW);  // also turn on blue LEDs to create purple
                    }
                    else  // cold condition --  blue LEDs
                    {
                        g_ioport.p_api->pinWrite(LED_pin[x][0], IOPORT_LEVEL_HIGH);  // turn off red LEDs
                        g_ioport.p_api->pinWrite(LED_pin[x][1], IOPORT_LEVEL_HIGH);  // turn off green LEDs
                        g_ioport.p_api->pinWrite(LED_pin[x][2], IOPORT_LEVEL_LOW);  // turn on blue LEDs
                    }
                }

                if(justplayed == 0)
                {
                    play_song_flag = 1;  //prevent the song from continuing to play while the drink is on the coaster
                }

            }
            if(justplayed == 1) // song has finished and beverage is still on caster
                            {
                                for (int x = 0; x<5; x++)
                                                {
                                                    if (coldness == 0)  // hot condition --  red LEDs
                                                    {
                                                        g_ioport.p_api->pinWrite(LED_pin[x][0], IOPORT_LEVEL_LOW);  // turn on red LEDs
                                                        g_ioport.p_api->pinWrite(LED_pin[x][1], IOPORT_LEVEL_HIGH);  // turn off green LEDs
                                                        g_ioport.p_api->pinWrite(LED_pin[x][2], IOPORT_LEVEL_HIGH);  // turn off blue LEDs
                                                    }
                                                    else if (coldness == 1)  // warming condition -- purple LEDs
                                                    {
                                                        g_ioport.p_api->pinWrite(LED_pin[x][0], IOPORT_LEVEL_LOW);  // turn on red LEDs
                                                        g_ioport.p_api->pinWrite(LED_pin[x][1], IOPORT_LEVEL_HIGH);  // turn off green LEDs
                                                        g_ioport.p_api->pinWrite(LED_pin[x][2], IOPORT_LEVEL_LOW);  // also turn on blue LEDs to create purple
                                                    }
                                                    else  // cold condition --  blue LEDs
                                                    {
                                                        g_ioport.p_api->pinWrite(LED_pin[x][0], IOPORT_LEVEL_HIGH);  // turn off red LEDs
                                                        g_ioport.p_api->pinWrite(LED_pin[x][1], IOPORT_LEVEL_HIGH);  // turn off green LEDs
                                                        g_ioport.p_api->pinWrite(LED_pin[x][2], IOPORT_LEVEL_LOW);  // turn on blue LEDs
                                                    }
                                                }

                            }
        }
        else
        {
            play_song();
        }
    }

}


void blink_rate_timer_callback(timer_callback_args_t * p_args)
{
    SSP_PARAMETER_NOT_USED (p_args);
    // Functions that happen when blink timer expires
    blink_on = 1;
    if (lowbattdetect ==1)
    {
        level ^= 1;
        blink_on = 0;
        for (int x = 0; x<5; x++)
        {
            g_ioport.p_api->pinWrite(LED_pin[x][0], IOPORT_LEVEL_HIGH);  // turn off LED
            g_ioport.p_api->pinWrite(LED_pin[x][1], IOPORT_LEVEL_HIGH);  // turn off LED
            g_ioport.p_api->pinWrite(LED_pin[x][2], IOPORT_LEVEL_HIGH);  // blink LED

        }
        g_ioport.p_api->pinWrite(LED_pin[0][0], level);  // blink red LED0

    }
}


void ambient_timer_callback(timer_callback_args_t * p_args)
{
    SSP_PARAMETER_NOT_USED (p_args);
    // Take an ambient light reading a few seconds after the coaster is turned on.
    if ( ambient_light_set_flag == 0 )
    {
        ambient_light = light_reading;
        ambient_light_set_flag = 1;
    }
}
/*
 * ******
 * * undelete this section if U1 is installed
 void g_lowbatt_irq0_callback(external_irq_callback_args_t * p_args)
{
    SSP_PARAMETER_NOT_USED (p_args);
    blink_rate_timer.p_api ->stop(blink_rate_timer.p_ctrl);
    blink_rate_timer.p_api -> periodSet(blink_rate_timer.p_ctrl, 300, TIMER_UNIT_PERIOD_MSEC);
    blink_rate_timer.p_api -> start(blink_rate_timer.p_ctrl);
    lowbattdetect = 1; // batteries are below 2.2V

    while(1)
    {
        // remain in loop when batteries are below 2.1v
        // blink_rate_timer will cause one Red LED to blink to indicate batteries need replacing
    }

}
*/


